# frozen_string_literal: true

require 'forwardable'

module LetItCrash
  module Builders
    class ReportBuilder
      extend Forwardable

      def_delegators :matcher, :matches?, :branch, :sha

      # @param result: result [SimpleCov::Result]
      # @param rewriter: rewriter [Proc]
      #
      # @return [LetItCrash::Proto::Report]
      def self.build(environment:, result:, rewriter:)
        new(environment: environment, result: result, rewriter: rewriter).build
      end

      def initialize(environment:, result:, rewriter:)
        @environment = environment
        @result = result
        @rewriter = rewriter
      end

      def build
        Proto::Report.new(commit: commit, files: files)
      end

      private

      attr_reader :environment, :result, :rewriter

      def commit
        raise MatcherMissing unless matches?
        Proto::Report::Commit.new(branch: branch, sha: sha)
      end

      def files
        result.files.map do |file|
          FileBuilder.build(file: file, rewriter: rewriter)
        end
      end

      def matcher
        @matcher ||= Matcher.new(environment: environment)
      end

      class MatcherMissing < RuntimeError; end
    end # class ReportBuilder
  end # module Builders
end # module LetItCrash
