# frozen_string_literal: true

require 'forwardable'

module LetItCrash
  class Matcher < Matchers::Base
    extend Forwardable

    def_delegators :matcher, :branch, :sha

    # This method performs a :reek:NilCheck.
    def matches?
      !matcher.nil?
    end

    private

    def matcher
      @matcher ||= MATCHERS.each do |klass|
        instance = klass.new(environment: environment)
        return instance if instance.matches?
      end
      nil
    end

    MATCHERS = [
      Matchers::AppVeyor,
      Matchers::Buildkite,
      Matchers::CircleCI,
      Matchers::Codefresh,
      Matchers::Codeship,
      Matchers::Drone,
      Matchers::GitLabCI,
      Matchers::Jenkins,
      Matchers::Semaphore,
      Matchers::Shippable,
      Matchers::SnapCI,
      Matchers::SolanoCI,
      Matchers::TeamCity,
      Matchers::TravisCI,
      Matchers::Wercker,
      Matchers::Git # This is a fallback.
    ].freeze
    private_constant :MATCHERS
  end # class Matcher
end # module LetItCrash
