# frozen_string_literal: true

require 'rest-client'

module LetItCrash
  module Reporters
    class Upload
      TOKEN_HEADER = 'X-LetItCrash-Token'

      def self.from_env
        endpoint = ENV.fetch(
          'LETITCRASH_COVERAGE_ENDPOINT',
          'https://notify.letitcrash.io/coverage/ruby',
        )
        new(endpoint: endpoint, token: ENV.fetch('LETITCRASH_COVERAGE_TOKEN'))
      end

      def initialize(endpoint:, token:)
        @endpoint = endpoint
        @token = token
      end

      def report(input)
        blockers_off
        RestClient.post(endpoint, input.to_json, headers)
        blockers_on
      end

      private

      attr_reader :endpoint, :token

      # This method reeks of :reek:UtilityFunction.
      def blockers_off
        WebMock.disable! if defined?(WebMock)
        VCR.turn_off!(ignore_cassettes: true) if defined?(VCR)
      end

      # This method reeks of :reek:UtilityFunction.
      def blockers_on
        WebMock.enable! if defined?(WebMock)
        VCR.turn_on! if defined?(VCR)
      end

      def headers
        {
          content_type: :json,
          accept: :json,
          TOKEN_HEADER => token
        }
      end
    end # class Upload
  end # module Reporters
end # module LetItCrash
