# frozen_string_literal: true

require 'forwardable'
require 'set'

require 'parser/current'

module LetItCrash
  class Sanitizer
    extend Forwardable

    def_delegators :@relevant, :include?

    def self.from_path(path:)
      from_source(source: File.read(path))
    end

    def self.from_source(source:)
      new(root: Parser::CurrentRuby.parse(source))
    end

    def initialize(root:)
      @relevant = Set.new
      visit(root)
    end

    private

    # This method reeks of :reek:FeatureEnvy.
    def visit(node)
      return unless node.is_a?(Parser::AST::Node)
      loc = node.loc
      @relevant.add(loc.first_line) if loc.expression
      node.children.each(&method(:visit))
    end
  end # class Sanitizer
end # module LetItCrash
